﻿using CodeSamples.Ch01_ServiceOrientedArchitecture.Listing1;

namespace CodeSamples.Ch01_ServiceOrientedArchitecture.Listing2
{
  /// <summary>
  /// An implementing class MUST provide the appropriate
  /// items described in the interface or the project will not build.
  /// The implementing class can add any additional 
  /// features deemed appropriate.
  /// </summary>
  public class Example : IExample
  {
    /// <summary>
    /// Implement the StringProperty described in the interface
    /// </summary>
    public string StringProperty { get; set; }

    /// <summary>
    /// The ReadOnlyStringProperty cannot have a public setter,
    /// but can have a private, protected, or internal setter.
    /// </summary>
    public string ReadOnlyStringProperty { get; internal set; }

    /// <summary>
    /// The implementing class must provide the method signatures
    /// described in the interface, though if no logic is
    /// added to the method, it will still build.
    /// </summary>
    public int OperateOnTwoNumbers(int pNum1, int pNum2)
    {
      return pNum1 + pNum2;
    }

    /// <summary>
    /// Any further customization of the implementing
    /// class is allowed
    /// </summary>
    public bool MethodNotInTheInterface()
    {
      return true;
    }
  }
}

