﻿using System;
using CodeSamples.Ch01_ServiceOrientedArchitecture.Listing1;

namespace CodeSamples.Ch01_ServiceOrientedArchitecture.Listing3
{
  /// <summary>
  /// An implementing class MUST provide the appropriate
  /// items described in the interface or the project will not build.
  /// The implementing class can add any additional 
  /// features deemed appropriate.
  /// </summary>
  public class Example2 : IExample
  {
    /// <summary>
    /// Implement the StringProperty described in the interface
    /// </summary>
    public string StringProperty { get; set; }

    /// <summary>
    /// The ReadOnlyStringProperty cannot have a public setter,
    /// but can have a private, protected, or internal setter.
    /// </summary>
    public string ReadOnlyStringProperty { get; internal set; }

    /// <summary>
    /// Implementing a method described in an interface does not
    /// require that proper logic be added, only that the method
    /// signature is provided.
    /// </summary>
    public int OperateOnTwoNumbers(int pNum1, int pNum2)
    {
      throw new NotImplementedException("OperateOnTwoNumbers");
    }

    /// <summary>
    /// Any further customization of the implementing
    /// class is allowed
    /// </summary>
    public bool MethodNotInTheInterface()
    {
      return true;
    }
  }
}

