﻿using System.Windows.Forms;

namespace CodeSamples.Ch01_ServiceOrientedArchitecture
{
  /// <summary>
  /// This class will retrieve a string from the database and
  /// display it on the screen
  /// </summary>
  public class ShowDatabaseInfo
  {
    /// <summary>
    /// Main driver method
    /// </summary>
    public void GetAndShowData(int recordId)
    {
      ADODB.Recordset data = GetData();
      DisplayForm frm = GetForm();
      ShowDataInForm(frm, data);
    }

    /// <summary>
    /// Open an ADO connection to Microsoft SQL Server,
    /// get the data and return it
    /// </summary>
    public ADODB.Recordset GetData()
    {
      var connection = new ADODB.Connection();
      connection.ConnectionString = "my connection string";
      connection.Open();
      var rs = new ADODB.Recordset();

      //The following query is specific to Microsoft SQL Server
      //it will not execute properly on Oracle
      rs.Open("Microsoft SQL Specific Query", connection);

      rs.ActiveConnection = null;
      return rs;
    }

    /// <summary>
    /// Get the proper form to use for our display
    /// </summary>
    private DisplayForm GetForm()
    {
      return new DisplayForm();
    }

    /// <summary>
    /// Show the information to the user in the given form
    /// </summary>
    public void ShowDataInForm(DisplayForm frm, ADODB.Recordset data)
    {
      frm.UserFirstNameLbl.Text = data.Fields[0].ToString();
      frm.UserLastNameLbl.Text = data.Fields[0].ToString();
      frm.Show();
    }
  }

  /// <summary>
  /// Form that displays the information to the user
  /// </summary>
  public class DisplayForm : Form
  {
    public Label UserFirstNameLbl { get; set; }
    public Label UserLastNameLbl { get; set; }

    /// <summary>
    /// constructor that creates the data display form
    /// </summary>
    public DisplayForm()
    {
      UserFirstNameLbl = new Label();
      Controls.Add(UserFirstNameLbl);

      UserLastNameLbl = new Label();
      Controls.Add(UserLastNameLbl);
    }

  }



}

