﻿using System.Windows.Forms;

namespace CodeSamples.Ch01_ServiceOrientedArchitecture
{
  /// <summary>
  /// This class will retrieve a string from the database and
  /// display it on the screen
  /// </summary>
  public class ShowDatabaseInfoAbstracted
  {
    /// <summary>
    /// Main driver method
    /// </summary>
    public void GetAndShowData(int recordId)
    {
      ADODB.Recordset data = GetData();
      DisplayFormAbstracted frm = GetForm();
      ShowDataInForm(frm, data);
    }

    /// <summary>
    /// Open an ADO connection to Microsoft SQL Server,
    /// get the data and return it
    /// </summary>
    public ADODB.Recordset GetData()
    {
      var connection = new ADODB.Connection();
      connection.ConnectionString = "my connection string";
      connection.Open();
      var rs = new ADODB.Recordset();

      //The following query is specific to Microsoft SQL Server
      //it will not execute properly on Oracle
      rs.Open("Microsoft SQL Specific Query", connection);

      rs.ActiveConnection = null;
      return rs;
    }

    /// <summary>
    /// Get the proper form to use for our display
    /// </summary>
    private DisplayFormAbstracted GetForm()
    {
      return new DisplayFormAbstracted();
    }

    /// <summary>
    /// Show the information to the user in the given form
    /// </summary>
    public void ShowDataInForm(DisplayFormAbstracted frm, ADODB.Recordset data)
    {
      frm.UserFirstName = data.Fields[0].ToString();
      frm.UserLastName = data.Fields[0].ToString();
      frm.Show();
    }
  }

  /// <summary>
  /// Form that displays the information to the user
  /// </summary>
  public class DisplayFormAbstracted : Form
  {
    /// <summary>
    /// This is private so it cannot be used by an outside
    /// consumer, thus improving the abstraction.
    /// </summary>
    private Label _userFirstNameLbl;
    private Label _userLastNameLbl;

    /// <summary>
    /// these getters and setters can change as the
    /// display requirements change, but the calling
    /// code will be unaffected
    /// </summary>
    public string UserFirstName
    {
      get { return _userFirstNameLbl.Text; }
      set { _userFirstNameLbl.Text = value; }
    }

    /// <summary>
    /// these getters and setters can change as the
    /// display requirements change, but the calling
    /// code will be unaffected
    /// </summary>
    public string UserLastName
    {
      get { return _userLastNameLbl.Text; }
      set { _userLastNameLbl.Text = value; }
    }

    /// <summary>
    /// constructor that creates the data display form
    /// </summary>
    public DisplayFormAbstracted()
    {
      _userFirstNameLbl = new Label();
      Controls.Add(_userFirstNameLbl);

      _userLastNameLbl = new Label();
      Controls.Add(_userLastNameLbl);
    }

  }



}

