﻿
using System.Windows.Forms;

namespace CodeSamples.Ch01_ServiceOrientedArchitecture
{
  /// <summary>
  /// Interface for a service that displays messages to the user
  /// </summary>
  public interface IDialogService
  {
    /// <summary>
    /// Show the given message to the user in a dialog
    /// with just an OK button
    /// </summary>
    void Show(string pUserMessage);

    /// <summary>
    /// Show a message to the user with the specified
    /// window caption and buttons
    /// </summary>
    DialogResult Show(string pUserMessage, string pCaption,
      MessageBoxButtons pMessageBoxButtons);
  }

  /// <summary>
  /// The dialog service will show a message to the user
  /// and accept an appropriate response as requested
  /// by the caller
  /// </summary>
  public class DialogService : IDialogService
  {
    /// <summary>
    /// Show the given message to the user in a dialog
    /// with just an OK button
    /// </summary>
    public void Show(string pUserMessage)
    {
      MessageBox.Show(pUserMessage);
    }

    /// <summary>
    /// Show a message to the user with the specified
    /// window caption and buttons
    /// </summary>
    public DialogResult Show(string pUserMessage, string pCaption,
      MessageBoxButtons pMessageBoxButtons)
    {
      return MessageBox.Show(pUserMessage, pCaption, pMessageBoxButtons);
    }
  }
}