﻿using Microsoft.Practices.Unity;

namespace CodeSamples.Ch04_DependencyInjectionPrinciple.Listing10
{
  /// <summary>
  /// Interface for the class that does the real work
  /// </summary>
  public interface IExampleClass
  {
    void DoWork();
  }

  /// <summary>
  /// Concrete implementation of the worker class
  /// </summary>
  public class ExampleClass : IExampleClass
  {
    public void DoWork()
    {
      //do work here
    }
  }


  public class IoCContainerExample
  {
    public void ExampleUsage()
    {
      //create the container so we can register our classes
      var container = new UnityContainer();

      //this is how we register our class with the container
      //this tells the container that when a caller requests
      //an IExampleClass, the container should create and
      //return an ExampleClass
      container.RegisterType<IExampleClass, ExampleClass>();

      //this is how we get an instance of the ExampleClass
      //We request an instance of the IExampleClass and
      //the container creates and returns an Exam-pleClass which
      //implements the IExampleClass interface
      var instance = container.Resolve<IExampleClass>();
    }

  }
}
  