﻿using Microsoft.Practices.Unity;

namespace CodeSamples.Ch04_DependencyInjectionPrinciple.Listing11
{
  /// <summary>
  /// Interface for the class that does the real work
  /// </summary>
  public interface IExampleClass
  {
    void DoWork();
  }

  /// <summary>
  /// Concrete implementation of the worker class
  /// </summary>
  public class ExampleClass : IExampleClass
  {
    public void DoWork()
    {
      //do work using another class
      var dependencyClass = new DependencyClass();
      dependencyClass.DoDependencyWork();
    }
  }

  public interface IDependencyClass
  {
    void DoDependencyWork();
  }

  public class DependencyClass : IDependencyClass
  {
    public void DoDependencyWork()
    {
      //do work
    }
  }

  public class IoCContainerExample
  {
    public void ExampleUsage()
    {
      var container = new UnityContainer();
      container.RegisterType<IExampleClass, ExampleClass>();
      var instance = container.Resolve<IExampleClass>();
    }

  }
}
  