﻿using Microsoft.Practices.Unity;

namespace CodeSamples.Ch04_DependencyInjectionPrinciple.Listing12
{
  /// <summary>
  /// Interface for the class that does the real work
  /// </summary>
  public interface IExampleClass
  {
    void DoWork();
  }

  /// <summary>
  /// Concrete implementation of the worker class
  /// </summary>
  public class ExampleClass : IExampleClass
  {
    public void DoWork()
    {
      //*** We are now using the ServiceLocator to resolve our dependency
      var dependencyClass = ServiceLocator.Resolve<IDependencyClass>();
      dependencyClass.DoDependencyWork();
    }
  }

  public interface IDependencyClass
  {
    void DoDependencyWork();
  }

  public class DependencyClass : IDependencyClass
  {
    public void DoDependencyWork()
    {
      //do work
    }
  }

  public class IoCContainerExample
  {
    public void ExampleUsage()
    {
      //*** we are now using the ServiceLocator to get
      //our example class
      var instance = ServiceLocator.Resolve<IExampleClass>();
    }

  }

  public static class ServiceLocator
  {
    private static UnityContainer _container;

    static ServiceLocator()
    {
      //Create our container
      _container = new UnityContainer();

      //setup all class registrations so they can be resolved
      //in code later
      _container.RegisterType<IExampleClass, ExampleClass>();
      _container.RegisterType<IDependencyClass, DependencyClass>();
    }

    public static TProxyType Resolve<TProxyType>()
    {
      return _container.Resolve<TProxyType>();
    }
  }
}
  