﻿
using Microsoft.Practices.Unity;

namespace CodeSamples.Ch04_DependencyInjectionPrinciple.Listing21
{
  public class Program
  {
    public Program()
    {
      var container = new UnityContainer();
      container.RegisterType<IParentClass, ParentClass>();
      container.RegisterType<IDependencyClass, DependencyClass>();
      container.RegisterType<IRepository, Repository>();

      var parent = container.Resolve<IParentClass>();
      parent.DoSomething();
    }
  }

  public interface IParentClass
  {
    void DoSomething();
  }

  public class ParentClass : IParentClass
  {
    private IDependencyClass _dependency;
    private IRepository _repo;

    public ParentClass(IDependencyClass dependencyClass, 
      IRepository repo)
    {
      _dependency = dependencyClass;
      _repo = repo;
    }

    public void DoSomething()
    {
      _dependency.DoSomethingElse();
      var data = _repo.GetData();
    }
  }

  public interface IDependencyClass
  {
    void DoSomethingElse();
  }
  public class DependencyClass : IDependencyClass
  {
    private IRepository _repo;
    public DependencyClass(IRepository repo)
    {
      _repo = repo;
    }
    public void DoSomethingElse()
    {
      //do the actual work
      var data = _repo.GetData();
    }
  }

  public interface IRepository
  {
    string GetData();
  }

  public class Repository : IRepository
  {
    public string  GetData()
    {
      return "data";
    }
  }
}