﻿using System.Windows;
using CodeSamples.Ch10_EstablishFoundation.Listing02;
using CodeSamples.Ch10_EstablishFoundation.Listing03;
using CodeSamples.Ch10_EstablishFoundation.Listing04;
using Microsoft.Practices.Prism.UnityExtensions;
using Microsoft.Practices.Unity;


namespace CodeSamples.Ch10_EstablishFoundation.Listing05
{
  /// <summary>
  /// This class will be created in the Shell project
  /// </summary>
  public class BootstrapperWinforms : UnityBootstrapper
  {

    /// <summary>
    /// This is the entry point into our custom bootstrapper
    /// and is called by the base
    /// </summary>
    protected override void ConfigureContainer()
    {
      base.ConfigureContainer();
      ServiceLocator.Container = Container;

      RegisterControlsAndControllers(Container);
      RegisterServices(Container);
    }

    /// <summary>
    /// Registers all services for the application
    /// </summary>
    protected void RegisterServices(IUnityContainer container)
    {
      //Service registrations will go here
    }

    /// <summary>
    /// Registers the views and controllers.
    /// </summary>
    protected void RegisterControlsAndControllers(IUnityContainer container)
    {
      //our shell should also be a singleton
      container.RegisterType<IShellForm, ShellForm>(
        new ContainerControlledLifetimeManager());
      container.RegisterType<IShellFormController, ShellFormController>(
        new ContainerControlledLifetimeManager());
    }

    /// <summary>
    /// Returns the controller for the shell so Program.cs
    /// can run the app shell.  This is Winforms specific
    /// and should be replaced in the WPF version
    /// </summary>
    public IShellFormController ShellController
    {
      get
      {
        var x = Container.Resolve<IShellFormController>();
        return x;
      }
    }

    /// <summary>
    /// This is a requirement by the UnityBootstrapper because
    /// it was built with WPF in mind.  For Winforms
    /// this makes no sense so we just return null.
    /// </summary>
    protected override DependencyObject CreateShell()
    {
      return null;
    }
  }

}