﻿using System.Windows;
using CodeSamples.Ch10_EstablishFoundation.Listing02;
using CodeSamples.Ch10_EstablishFoundation.Listing04;
using CodeSamples.Ch10_EstablishFoundation.Listing07;
using CodeSamples.Ch10_EstablishFoundation.Listing09;
using Microsoft.Practices.Prism.UnityExtensions;
using Microsoft.Practices.Unity;

namespace CodeSamples.Ch10_EstablishFoundation.Listing08
{
  public class BootstrapperWPF : UnityBootstrapper
  {
    private IShellFormViewModel _shellVM;

    protected override void ConfigureContainer()
    {
      base.ConfigureContainer();
      ServiceLocator.Container = Container;

      RegisterControlsAndControllers(Container);
      RegisterServices(Container);
    }


    /// <summary>
    /// Registers services for the application
    /// </summary>
    protected void RegisterServices(IUnityContainer container)
    {
      //Service registration will go here, for example
      //container.RegisterType<ILogger, Logger>(
      //  new ContainerControlledLifetimeManager());
    }


    /// <summary>
    /// Registers the views and view models.
    /// </summary>
    protected void RegisterControlsAndControllers(IUnityContainer container)
    {
      //our shell should also be a singleton
      container.RegisterType<IShellForm, ShellForm>(
        new ContainerControlledLifetimeManager());
      container.RegisterType<IShellFormViewModel, ShellFormViewModel>(
        new ContainerControlledLifetimeManager());
    }



    /// <summary>
    /// Initializes the shell.  This is WPF specific and should be
    /// replaced in the winforms version.
    /// </summary>
    /// <remarks>
    /// This runs after CreateShell
    /// </remarks>
    protected override void InitializeShell()
    {
      base.InitializeShell();

      _shellVM.Activate();

      App.Current.MainWindow = (Window)Shell;
      App.Current.MainWindow.Show();
    }

    /// <summary>
    /// Creates the application shell.  This is WPF specific and should
    /// be replaced in the Winforms version.
    /// </summary>
    /// <remarks>
    /// This runs before InitializeShell
    /// </remarks>
    protected override DependencyObject CreateShell()
    {
      _shellVM = ServiceLocator.Resolve<IShellFormViewModel>();
      return _shellVM.View as DependencyObject;
    }
  }

}