﻿using System;
using System.Collections.Generic;

//This example shows how we can have two different services
//that each implement the same interface, then at runtime
//decide which service is registered for use.
namespace CodeSamples.Ch10_EstablishFoundation.Listing10
{
  public enum Currency
  {
    Dollar,
    Euro,
    Yen
  }

  public interface ICurrencyConverter
  {
    decimal Convert(Currency destinationCurrency, decimal sourceAmt);
  }

  /// <summary>
  /// Very simple currency converter available only to
  /// those customers that purchase it.
  /// </summary>
  public class CurrencyConverter : ICurrencyConverter
  {
    private Dictionary<Currency, decimal> _conversionRates;

    public CurrencyConverter()
    {
      _conversionRates = new Dictionary<Currency, decimal>();
      //add code to fill the _conversionRates dictionary
    }

    public decimal Convert(Currency destinationCurrency, decimal sourceAmt)
    {
      var conversionFactor = _conversionRates[destinationCurrency];
      return conversionFactor * sourceAmt;
    }
  }

  /// <summary>
  /// Dummy currency converter that is provided to customers
  /// that do not purchase the fully functional converter.
  /// </summary>
  public class CurrencyConverterDummy : ICurrencyConverter
  {
    public decimal Convert(Currency destinationCurrency, decimal sourceAmt)
    {
      if (destinationCurrency != Currency.Dollar) 
        throw new ArgumentException("Please purchase currency module");
      return sourceAmt;
    }
  }
}
