﻿using System;
using CodeSamples.Ch10_EstablishFoundation.Listing04;

namespace CodeSamples.Ch11_RefactorToBasicServices.Listing10
{
  public class BusinessLogic
  {
    public void SampleCallToApplicationSession()
    {
      var svc = ServiceLocator.Resolve<IApplicationSession>();
      var usrName = svc.UserName;
    }
  }

  /// <summary>
  /// This class that contains application session 
  /// information which may
  /// be used throughout the application.
  /// </summary>
  public class ApplicationSession : IApplicationSession
  {
    /// <summary>
    /// Holds the username for the currently logged in user
    /// </summary>
    public string UserName { get; internal set; }

    /// <summary>
    /// Holds the ID for the customer currently logged in
    /// </summary>
    public Guid CurrentCustomerId { get; internal set; }

    public ApplicationSession()
    {
      //this is where data would be populated, possibly
      //from the database or other permanent store
      UserName = "CurrentUser";
      CurrentCustomerId = Guid.NewGuid();
    }
  }

  public interface IApplicationSession
  {
    string UserName { get; }
    Guid CurrentCustomerId { get; }
  }

}
