﻿using ADODB;
using CodeSamples.Ch10_EstablishFoundation.Listing04;

namespace CodeSamples.Ch12_RefactorToAdvancedServices.Listing02
{
  public class SecurityService
  {
    public bool ValidateUserPassword(int userId, string password)
    {
      var repo = ServiceLocator.Resolve<IRepository>();
      var rs = repo.GetUserData(userId);

      //if locked out, fail
      if (rs.Fields["isLockedOut"].Value == "1") return false;

      //if password matches, succeed
      var validPW = (password == rs.Fields["password"].Value);

      //if last login was successful, no update needed
      var numBadLogins = int.Parse(rs.Fields["numBadLogins"].Value);
      var isLockedOut = int.Parse(rs.Fields["isLockedOut"].Value);
      if (validPW && numBadLogins == 0 && isLockedOut != 1) return true;

      //lock out user after 3 tries
      if (validPW)
      {
        rs.Fields["numBadLogins"].Value = 0;
        rs.Fields["isLockedOut"].Value = 0;
      }
      else
      {
        rs.Fields["numBadLogins"].Value = numBadLogins + 1;
        if (numBadLogins >= 3)
          rs.Fields["isLockedOut"].Value = 1;
      }
      repo.SaveUserData(rs);

      return false;
    }

  }

  public interface IRepository
  {
    Recordset GetUserData(int userId);
    void SaveUserData(Recordset userData);
  }

  public class Repository : IRepository
  {
    public Recordset GetUserData(int userId)
    {
      var conn = new Connection();
      conn.ConnectionString = "connection string";
      conn.Open();

      var qryString = "select * from User where id = " + userId;

      var rs = new Recordset();
      rs.CursorLocation = CursorLocationEnum.adUseClient;
      rs.CursorType = CursorTypeEnum.adOpenForwardOnly;
      rs.LockType = LockTypeEnum.adLockOptimistic;
      rs.Open(conn, qryString);
      rs.ActiveConnection = null;
      return rs;
    }

    public void SaveUserData(Recordset userData)
    {
      var updString = "update User set ";
      updString += " numBadLogins = " + userData.Fields[1].Value;
      updString += " ,isLockedOut = " + userData.Fields[2].Value;
      updString += " where id = " + userData.Fields[3].Value;

      var conn = new Connection();
      conn.ConnectionString = "connection string";
      conn.Open();

      var cmd = new Command();
      cmd.CommandType = CommandTypeEnum.adCmdText;
      cmd.CommandText = updString;
      cmd.ActiveConnection = conn;
      object recordsAffected;
      cmd.Execute(out recordsAffected, null);     
    }
  }
}