﻿using System;
using Microsoft.Practices.EnterpriseLibrary.Validation;
using Microsoft.Practices.EnterpriseLibrary.Validation.Validators;

namespace CodeSamples.Ch12_RefactorToAdvancedServices.Listing08
{
  [HasSelfValidation]
  public class Person : BaseEntity
  {
    public int Id { get; set; }
    public string FirstName { get; set; }
    public string LastName { get; set; }
    public DateTime BirthDate { get; set; }
    public string Email { get; set; }
    public int Age
    {
      get
      {
        DateTime now = DateTime.Today;
        int age = now.Year - BirthDate.Year;
        if (BirthDate > now.AddYears(-age)) age--;
        return age;
      }
    }
    public DateTime DateJoined { get; set; }
    public bool MembershipCancelled { get; set; }

    [SelfValidation]
    public void Validation_LastName_IsNotNull(
      ValidationResults results)
    {
      if (string.IsNullOrEmpty(LastName))
        results.AddResult(CreateValidationResult(
          "Lastname cannot be null."));
    }

    private ValidationResult CreateValidationResult(string errMsg)
    {
      var v = new ValidationResult(errMsg, this, 
        Id.ToString(), "Person", null);
      return v;
    }
  }

  public abstract class BaseEntity
  {
    public DateTime CreatedDate { get; set; }
    public DateTime UpdatedDate { get; set; }
  }
}