﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using CodeSamples.Ch10_EstablishFoundation.Listing04;
using CodeSamples.Ch12_RefactorToAdvancedServices.Listing04;
using Microsoft.Practices.EnterpriseLibrary.Validation;

namespace CodeSamples.Ch12_RefactorToAdvancedServices.Listing09
{
  public class Repository : IRepository
  {
    private ReengineeringEntities _context;
    private IValidationService ValidationSvc
    {
      get
      {
        return _validationSvc ??
          (_validationSvc = ServiceLocator.Resolve<IValidationService>());
      }
    }
    private IValidationService _validationSvc;

    public Repository()
    {
      _context = new ReengineeringEntities();
    }

    /// <summary>
    /// Verifies all edited items, then saves 
    /// all pending changes to the database
    /// </summary>
    public ValidationResults SaveChanges()
    {
      try
      {
        //Validate data and exit if invalid
        var results = Validate();
        if (!results.IsValid) return results;

        //Save changes to DB
        _context.SaveChanges();

        return results;
      }
      catch (Exception e)
      {
        //add logic to handle data save errors
        throw;
      }
    }

    /// <summary>
    /// Validates all changed entities in this Repository
    /// </summary>
    public ValidationResults Validate()
    {
      var entities = GetUpdatedItems();
      return ValidationSvc.Validate(entities);
    }

    /// <summary>
    /// Returns a list of all updated entities
    /// </summary>
    internal List<BaseEntity> GetUpdatedItems()
    {
      var allEntities = _context.ChangeTracker.Entries()
        .Where(e => e.State == EntityState.Added
          || e.State == EntityState.Modified)
        .Select(e => e.Entity)
        .OfType<BaseEntity>().ToList();
      return allEntities;
    }

  }

  public class ValidationService : IValidationService
  {
    public ValidationResults Validate(ICollection objectsToValidate)
    {
      var validationResults = new ValidationResults();
      foreach (var obj in objectsToValidate)
      {
        var validator = ValidationFactory.CreateValidator(obj.GetType());
        validationResults.AddAllResults(validator.Validate(obj));
      }
      return validationResults;
    }
  }

  public interface IRepository
  {
    ValidationResults SaveChanges();
    ValidationResults Validate();
  }

  public interface IValidationService
  {
    ValidationResults Validate(ICollection objectsToValidate);
  }


}