﻿using System;

namespace CodeSamples.Ch12_RefactorToAdvancedServices.Listing10
{
  public interface IObjectManagerReference<T> where T : class
  {
    T Object { get; }
    bool IsAlive { get; }
    string Name { get; }
  }

  public class StrongReferenceDto<T> 
    : IObjectManagerReference<T> where T : class
  {
    public virtual T Object { get; internal set; }
    public virtual bool IsAlive { get { return true; } }
    public string Name { get; internal set; }
    public StrongReferenceDto(T pObject, string pName)
    {
      if (pObject == null) throw new ArgumentNullException("pObject");
      Object = pObject;
      Name = pName;
    }
  }

  public class WeakReferenceDto<T> : 
    IObjectManagerReference<T> where T : class
  {
    private WeakReference _reference;

    public string Name { get; internal set; }
    public T Object
    {
      get { return _reference.Target as T; }
      internal set { _reference = new WeakReference(value); }
    }

    public bool IsAlive
    {
      get { return _reference.IsAlive; }
    }

    public WeakReferenceDto(T pObject, string pName)
    {
      if (pObject == null) throw new ArgumentNullException("pObject");
      Object = pObject;
      Name = pName;
    }
  }
}