﻿using System;

namespace CodeSamples.Ch12_RefactorToAdvancedServices.Listing12
{
  /// <summary>
  /// Base class for a command that is executed via the CommandDispatcher
  /// </summary>
  public abstract class BaseDispatchedCommand<TCommandKeyType> : IDispatchedCommand<TCommandKeyType>
  {
    public BaseDispatchedCommand(TCommandKeyType pCommandKey)
    {
      CommandKey = pCommandKey;
    }

    /// <summary>
    /// Determines whether this command can be executed.  Assume
    /// it is always available.
    /// </summary>
    public virtual bool CanExecute(object parameter)
    {
      return true;
    }

    /// <summary>
    /// Executes this command with the given parameter
    /// </summary>
    public abstract void Execute(object parameter);

    /// <summary>
    /// Notifies subscribers that the status of CanExecute has changed.
    /// </summary>
    public event EventHandler CanExecuteChanged;

    /// <summary>
    /// Key used to lookup this command.
    /// </summary>
    public TCommandKeyType CommandKey { get; internal set; }

    /// <summary>
    /// Number of times this command has been executed.
    /// </summary>
    public int ExecuteCount { get; set; }

  }

  internal class Cmd1Command : BaseDispatchedCommand<string>
  {
    //give the base the string used to invoke this command
    public Cmd1Command() : base("/cmd1") { }
    public override void Execute(object parameter)
    {
      //do something important
    }
  }

  public interface IDispatchedCommand<T>
  {
    T CommandKey { get; }
    bool CanExecute(Object parameter);
    void Execute(Object parameter);
    event EventHandler CanExecuteChanged;
    int ExecuteCount { get; set; }
  }

}