﻿using System;
using System.Windows.Forms;
using CodeSamples.Ch10_EstablishFoundation.Listing04;

namespace CodeSamples.Ch13_RefactorToViewModel.Listing01
{
  public class ShellForm : Form
  {
    private Button _showViewBtn;

    public ShellForm()
    {
      _showViewBtn = new Button();
      Controls.Add(_showViewBtn);
      _showViewBtn.Click += ShowViewRequested;
    }

    private void ShowViewRequested(object sender, System.EventArgs e)
    {
      var frm = new DataEntryView();
      frm.LoadUserData(1);
      frm.Show();
    }
  }

  public class DataEntryView : Form
  {
    private TextBox _userName;
    private User _user;
    private Button _saveBtn;

    public DataEntryView()
    {
      _saveBtn = new Button();
      Controls.Add(_saveBtn);
      _userName = new TextBox();
      Controls.Add(_userName);

      _saveBtn.Click += _saveBtn_Click;
    }

    void _saveBtn_Click(object sender, EventArgs e)
    {
      if (string.IsNullOrEmpty(_userName.Text))
      {
        var dlgSvc = ServiceLocator.Resolve<IDialogService>();
        dlgSvc.Show("You cannot leave name blank");
        return;
      }

      _user.Name = _userName.Text;
      var repository = ServiceLocator.Resolve<IRepository>();
      repository.SaveUser(_user);
    }

    public void LoadUserData(int userId)
    {
      var repository = ServiceLocator.Resolve<IRepository>();
      _user = repository.GetUser(userId);
      _userName.Text = _user.Name;
    }
  }

  public class User
  {
    public int Id { get; set; }
    public string Username { get; set; }
    public string Name { get; set; }
  }

  public interface IRepository
  {
    User GetUser(int id);
    void SaveUser(User u);
  }

  public interface IDialogService
  {
    void Show(string msg);
  }

  public class DialogService : IDialogService
  {
    public void Show(string msg)
    {
      MessageBox.Show(msg);
    }
  }

  public class Repository : IRepository
  {
    public User GetUser(int id)
    {
      //this code should retrieve the user from the database
      var u = new User();
      u.Id = id;
      u.Name = "Joe User";
      u.Username = "JoeUser";
      return u;
    }

    public void SaveUser(User u)
    {
      //code for saving the user to the DB goes here
    }
  }
}