﻿using System.Windows.Forms;
using CodeSamples.Ch10_EstablishFoundation.Listing04;

namespace CodeSamples.Ch13_RefactorToViewModel.Listing02
{
  public class ShellForm : Form
  {
    private Button _showViewBtn;

    public ShellForm()
    {
      _showViewBtn = new Button();
      Controls.Add(_showViewBtn);
      _showViewBtn.Click += ShowViewRequested;
    }

    private void ShowViewRequested(object sender, System.EventArgs e)
    {
      var frm = ServiceLocator.Resolve<IDataEntryView>();
      frm.LoadUserData(1);
      frm.Show();
    }
  }

  public interface IDataEntryView
  {
    void LoadUserData(int userId);
    void Show();
  }

  public class DataEntryView : Form, IDataEntryView
  {
    private TextBox _userName;
    private User _user;
    private Button _saveBtn;

    public DataEntryView()
    {
      _saveBtn = new Button();
      Controls.Add(_saveBtn);
      _userName = new TextBox();
      Controls.Add(_userName);

      _saveBtn.Click += _saveBtn_Click;
    }

    void _saveBtn_Click(object sender, System.EventArgs e)
    {
      if (string.IsNullOrEmpty(_userName.Text))
      {
        var dlgSvc = ServiceLocator.Resolve<IDialogService>();
        dlgSvc.Show("You cannot leave name blank");
        return;
      }

      _user.Name = _userName.Text;
      var repository = ServiceLocator.Resolve<IRepository>();
      repository.SaveUser(_user);
    }

    public void LoadUserData(int userId)
    {
      var repository = ServiceLocator.Resolve<IRepository>();
      _user = repository.GetUser(userId);
      _userName.Text = _user.Name;
    }
  }


  public class User
  {
    public string Username { get; set; }
    public string Name { get; set; }
  }

  public interface IRepository
  {
    User GetUser(int id);
    void SaveUser(User u);
  }

  public interface IDialogService
  {
    void Show(string msg);
  }
}