﻿using System;
using System.Windows.Forms;
using CodeSamples.Ch10_EstablishFoundation.Listing04;

namespace CodeSamples.Ch13_RefactorToViewModel.Listing04
{
 
  public class DataEntryView : Form, IDataEntryView
  {
    private TextBox _userName;
    private User _user;
    private Button _saveBtn;

    protected virtual void OnSaveRequested(object sender, EventArgs e)
    {
      var handler = SaveRequested;
      if (handler != null) handler(this, e);
    }
    public event EventHandler<EventArgs> SaveRequested;

    public DataEntryView()
    {
      _saveBtn = new Button();
      Controls.Add(_saveBtn);
      _userName = new TextBox();
      Controls.Add(_userName);

      _saveBtn.Click += OnSaveRequested;
    }

    public void LoadUserData(User currentUser)
    {
      _user = currentUser;
      //bind the user data to the necessary controls
    }
  }

  public class DataEntryViewController : IDataEntryViewController
  {
    public IDataEntryView View { get; internal set; }
    private User _currentUser;

    public DataEntryViewController()
    {
      View = ServiceLocator.Resolve<IDataEntryView>();
      this.View.SaveRequested += View_SaveRequested;
    }

    void View_SaveRequested(object sender, EventArgs e)
    {
      if (string.IsNullOrEmpty(_currentUser.Name))
      {
        var dlgSvc = ServiceLocator.Resolve<IDialogService>();
        dlgSvc.Show("You cannot leave name blank");
        return;
      }

      var repository = ServiceLocator.Resolve<IRepository>();
      repository.SaveUser(_currentUser);
    }

    public void LoadUserData(int userId)
    {
      var repository = ServiceLocator.Resolve<IRepository>();
      _currentUser = repository.GetUser(userId);
    }

    public void Show()
    {
      View.Show();
    }
  }

  public class User
  {
    public string Username { get; set; }
    public string Name { get; set; }
  }

  public interface IRepository
  {
    User GetUser(int id);
    void SaveUser(User u);
  }

  public interface IDialogService
  {
    void Show(string msg);
  }

  public interface IDataEntryView
  {
    void LoadUserData(User currentUser);
    void Show();
    event EventHandler<EventArgs> SaveRequested;
  }

  public interface IDataEntryViewController
  {
    void LoadUserData(int userId);
    void Show();
  }


}