﻿using System;

namespace CodeSamples.Ch03_UnitTesting.Listing8
{
  public class FinancialFunctions
  {
    /// <summary>
    /// P = (Pv*R) / [1 - (1 + R)^(-n)] 
    /// where
    ///     Pv  = Present Value (beginning 
    ///           value or amount of loan)
    ///     APR = Annual Percentage Rate 
    ///           (one year time period)
    ///     R   = Periodic Interest Rate = 
    ///           APR/ # of interest periods per year
    ///     P   = Monthly Payment
    ///     n   = # of interest periods for overall 
    ///           time period (i.e., interest
    ///           periods per year * number of years)
    /// </summary>
    /// <param name="pLoanAmount">Original amount of the loan</param>
    /// <param name="pYearlyInterestRate">Yearly interest rate</param>
    /// <param name="pNumMonthlyPayments">Num of mthly payments</param>
    /// <returns></returns>
    public double CalcLoanPayment(double pLoanAmount,
      double pYearlyInterestRate, int pNumMonthlyPayments)
    {
      //start code change *******************************
      if (pYearlyInterestRate < 0) 
        throw new ArgumentException(
          "pYearlyInterestRate cannot be negative");
      //end code change *******************************

      var mthlyInterestRate = pYearlyInterestRate / 1200;
      var numerator = pLoanAmount * mthlyInterestRate;
      var denominator = (1 - Math.Pow(1 + mthlyInterestRate,
            -pNumMonthlyPayments));
      var mthlyPayment = Math.Round(numerator / denominator, 2);
      return mthlyPayment;
    }

  }

}
