﻿
using System;
using System.Data.SqlClient;
using Microsoft.Practices.Unity;

namespace CodeSamples.Ch05_ClassMocking.Listing01
{
  public class MockingExample
  {
    public void UpdateUserData()
    {
      var repo = ServiceLocator.Resolve<IRepository>();
      var user = repo.GetUserData(1);
      user.Name = "New Name";
      repo.SaveUserData(user);
    }
  }

  public class User
  {
    public int Id { get; internal set; }
    public string Name { get; set; }
    public DateTime LastUpdateDate { get; set; }
    public DateTime CreateDate { get; set; }

    public User()
    {
      CreateDate = DateTime.Now;
      LastUpdateDate = DateTime.Now;
    }
  }

  public interface IRepository
  {
    User GetUserData(int pUserId);
    void SaveUserData(User pUser);
  }

  public class Repository : IRepository
  {
    private string _connectionString = "connStringGoesHere";

    public User GetUserData(int pUserId)
    {
      var qryString = "select * from User where id = @id";
      var conn = new SqlConnection(_connectionString);
      var cmd = new SqlCommand(qryString, conn);
      cmd.Parameters.AddWithValue("@id", pUserId);
      try
      {
        conn.Open();
        var reader = cmd.ExecuteReader();
        reader.Read();
        var usr = new User
        {
          Id = (int)reader[0],
          Name = reader[1].ToString(),
          CreateDate = DateTime.Parse( reader[2].ToString()),
          LastUpdateDate = DateTime.Parse( reader[3].ToString())
        };
        reader.Close();
        return usr;
      }
      catch (Exception)
      {
        //do something with the exception
      }
      return null;
    }

    public void SaveUserData(User pUser)
    {
      var qryString = "Update user set name = @username where id = @id";
      var conn = new SqlConnection(_connectionString);
      var cmd = new SqlCommand(qryString, conn);
      pUser.LastUpdateDate = DateTime.Now;
      cmd.Parameters.AddWithValue("@id", pUser.Id);
      cmd.Parameters.AddWithValue("@username", pUser.Name);
      cmd.Parameters.AddWithValue("@CreateDate", pUser.CreateDate);
      cmd.Parameters.AddWithValue("@LastUpdateDate", pUser.LastUpdateDate);
      try
      {
        conn.Open();
        cmd.ExecuteNonQuery();
      }
      catch (Exception)
      {
        //do something with the exception
      }
    }
  }

  public static class ServiceLocator
  {
    private static UnityContainer _container;

    static ServiceLocator()
    {
      _container = new UnityContainer();
      _container.RegisterInstance<IUnityContainer>(_container);
      _container.RegisterType<IRepository, Repository>();
    }

    public static TProxyType Resolve<TProxyType>()
    {
      return _container.Resolve<TProxyType>();
    }
  }

}

