﻿using System;
using System.Windows.Forms;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace CodeSamples.Ch11_RefactorToBasicServices.Listing05
{
  public class BusinessLogicClass
  {
    /// <summary>
    /// Example of a method that can be refactored to use
    /// the dialog service.
    /// </summary>
    public bool ValidateDateRange(DateTime pStartDate, DateTime pEndDate)
    {
      if (pEndDate < pStartDate)
      {
        MessageBox.Show("End Date must be after Start Date");
        return false;
      }
      return true;
    }
  }
  
  [TestClass]
  public class MockingExampleTestClass
  {
    static BusinessLogicClass _bizLogic;

    [ClassInitialize]
    public static void ClassInitialize(TestContext context)
    {
      _bizLogic = new BusinessLogicClass();
    }

    [TestMethod]
    public void ValidateDateRange_ValidDates_NoMessage()
    {
      var startDt = new DateTime(2000, 1, 1);
      var endDt = new DateTime(2000, 1, 2);
      var result = _bizLogic.ValidateDateRange(startDt, endDt);

      //we can only assume no message was shown if the method
      //call returns to us
      Assert.IsTrue(result);
    }

    [TestMethod]
    public void ValidateDateRange_SameDates_NoMessage()
    {
      var startDt = new DateTime(2000, 1, 2);
      var result = _bizLogic.ValidateDateRange(startDt, startDt);

      //we can only assume no message was shown if the method
      //call returns to us
      Assert.IsTrue(result);
    }

    [TestMethod]
    public void ValidateDateRange_InvalidDates_ShowsMessage()
    {
      var startDt = new DateTime(2000, 1, 2);
      var endDt = new DateTime(2000, 1, 1);
      //this will show a dialog box on screen
      var result = _bizLogic.ValidateDateRange(startDt, endDt);
      Assert.IsFalse(result);
    }
  }
}