﻿using ADODB;
using CodeSamples.Ch10_EstablishFoundation.Listing04;
using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;

namespace CodeSamples.Ch11_RefactorToBasicServices.Listing05
{
  public class SecurityService
  {
    public bool ValidateUserPassword(int userId, string password)
    {
      var repo = ServiceLocator.Resolve<IRepository>();
      var rs = repo.GetUserData(userId);

      //if locked out, fail
      if (rs.Fields["isLockedOut"].Value == 1) return false;

      //if password matches, succeed
      var validPW = (password == rs.Fields["password"].Value);

      //if last login was successful, no update needed
      int numBadLogins = rs.Fields["numBadLogins"].Value;
      int isLockedOut = rs.Fields["isLockedOut"].Value;
      if (validPW && numBadLogins == 0 && isLockedOut != 1) return true;

      //lock out user after 3 tries
      if (validPW)
      {
        rs.Fields["numBadLogins"].Value = 0;
        rs.Fields["isLockedOut"].Value = 0;
      }
      else
      {
        rs.Fields["numBadLogins"].Value = numBadLogins + 1;
        if (numBadLogins >= 3)
          rs.Fields["isLockedOut"].Value = 1;
      }
      repo.SaveUserData(rs);

      return false;
    }

  }

  public interface IRepository
  {
    Recordset GetUserData(int userId);
    void SaveUserData(Recordset userData);
  }

  [TestClass]
  public class SecurityServiceTestClass
  {
    private SecurityService _svc;
    private Mock<IRepository> _mockRepo;

    [TestInitialize]
    public void TestInitialize()
    {
      _mockRepo = new Mock<IRepository>();
      ServiceLocator.Container = new UnityContainer();
      ServiceLocator.RegisterInstance<IRepository>(
        _mockRepo.Object);

      _svc = new SecurityService();
    }

    [TestMethod]
    public void ValidateUserPassword_WhenValidPassword_ReturnsTrue()
    {
      //Arrange
      var rs = CreateNewUserRecordset();
      AddDataToUserRecordset(rs, 1, "password", 0, 0);
      _mockRepo.Setup(mr => mr.GetUserData(It.IsAny<int>()))
        .Returns(rs);

      //Act
      var result = _svc.ValidateUserPassword(1, "password");

      //assert
      Assert.IsTrue(result);
    }

    [TestMethod]
    public void ValidateUserPassword_WhenBadPassword_ReturnsFalse()
    {
      //Arrange
      var rs = CreateNewUserRecordset();
      AddDataToUserRecordset(rs, 1, "password", 0, 0);
      _mockRepo.Setup(mr => mr.GetUserData(It.IsAny<int>()))
        .Returns(rs);

      //Act
      var result = _svc.ValidateUserPassword(1, "badpassword");

      //assert
      Assert.IsFalse(result);
    }

    private void AddDataToUserRecordset(Recordset rs, int id, 
      string password, int numBadLogins, int isLockedOut)
    {
      rs.AddNew( );
      rs.Fields["id"].Value = id;
      rs.Fields["password"].Value = password;
      rs.Fields["numBadLogins"].Value = numBadLogins;
      rs.Fields["isLockedOut"].Value = isLockedOut;
    }
    private Recordset CreateNewUserRecordset()
    {
      var rs = new Recordset();
      rs.CursorType = CursorTypeEnum.adOpenDynamic;
      rs.Fields.Append("id", DataTypeEnum.adInteger);
      rs.Fields.Append("password", DataTypeEnum.adVarChar, 250);
      rs.Fields.Append("numBadLogins", DataTypeEnum.adInteger);
      rs.Fields.Append("isLockedOut", DataTypeEnum.adInteger);
      rs.Open();
      return rs;
    }

  }
}